/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName: Customizable_Columns.sql 
*
*   Summary: Create Event Session to demonstrate using customizable columns.
*
*   Date: October 7, 2011 
*
*   SQL Server Versions:
*         2012
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

-- Look at available columns
SELECT 
	o.name AS event_name, 
	o.description,
	oc.name AS column_name, 
	column_value, 
	oc.description
FROM sys.dm_xe_packages AS p
JOIN sys.dm_xe_objects AS o 
     ON p.guid = o.package_guid
JOIN sys.dm_xe_object_columns AS oc 
	ON o.package_guid = oc.object_package_guid
		AND o.name = oc.object_name
WHERE (p.capabilities IS NULL OR p.capabilities & 1 = 0)
  AND (o.capabilities IS NULL OR o.capabilities & 1 = 0)
  AND (oc.capabilities IS NULL OR oc.capabilities & 1 = 0)
  AND object_type = 'event'
  AND column_type = 'customizable'
ORDER BY event_name;



-- If the Event Session exists Drop it
IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_CustomizableColumns')
	DROP EVENT SESSION [SQLskills_CustomizableColumns] ON SERVER;

-- Create Event Session to find the database with most SP Recompile Events
CREATE EVENT SESSION [SQLskills_CustomizableColumns]
ON SERVER
ADD EVENT sqlserver.existing_connection
ADD TARGET package0.ring_buffer;
GO

ALTER EVENT SESSION [SQLskills_CustomizableColumns]
ON SERVER
STATE=START;
GO

SELECT 'ABC123';
GO

ALTER EVENT SESSION [SQLskills_CustomizableColumns]
ON SERVER
DROP EVENT sqlserver.existing_connection
GO

SELECT TOP 1 
	td.query('.') as n,
	td.value('(data[@name="options_text"]/value)[1]', 'nvarchar(max)') AS options_text
FROM 
(
    SELECT CAST(target_data AS XML) AS target_data
    FROM sys.dm_xe_sessions AS s    
    JOIN sys.dm_xe_session_targets AS t
        ON s.address = t.event_session_address
    WHERE s.name = 'SQLskills_CustomizableColumns'
        AND t.target_name = 'ring_buffer'
) AS sub
CROSS APPLY target_data.nodes('RingBufferTarget/event') AS q(td);
GO

-- Drop the Event Session
DROP EVENT SESSION [SQLskills_CustomizableColumns] 
ON SERVER;
GO

-- Create Event Session to demonstrate using a customizable column
CREATE EVENT SESSION [SQLskills_CustomizableColumns]
ON SERVER
ADD EVENT sqlserver.existing_connection(
	SET collect_options_text = 1)
ADD TARGET package0.ring_buffer;
GO


ALTER EVENT SESSION [SQLskills_CustomizableColumns]
ON SERVER
STATE=START;
GO

SELECT 'ABC123';
GO

ALTER EVENT SESSION [SQLskills_CustomizableColumns]
ON SERVER
DROP EVENT sqlserver.existing_connection;
GO

SELECT TOP 1 
	td.query('.') as n,
	td.value('(data[@name="options_text"]/value)[1]', 'nvarchar(max)') AS options_text
FROM 
(
    SELECT CAST(target_data AS XML) AS target_data
    FROM sys.dm_xe_sessions AS s    
    JOIN sys.dm_xe_session_targets AS t
        ON s.address = t.event_session_address
    WHERE s.name = 'SQLskills_CustomizableColumns'
        AND t.target_name = 'ring_buffer'
) AS sub
CROSS APPLY target_data.nodes('RingBufferTarget/event') AS q(td)





