/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName: Using_Actions.sql 
*
*   Summary: Create Event Session to demonstrate using customizable columns.
*
*   Date: October 7, 2011 
*
*   SQL Server Versions:
*         2012
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/



-- List all Actions
SELECT p.name AS package_name,
        o.name AS source_name,
        o.description
FROM sys.dm_xe_objects AS o
JOIN sys.dm_xe_packages AS p
      ON o.package_guid = p.guid
WHERE (p.capabilities IS NULL OR p.capabilities & 1 = 0)
   AND (o.capabilities IS NULL OR o.capabilities & 1 = 0)
   AND o.object_type = 'action';


/*************** Using the tsql_stack ***************/

IF EXISTS (SELECT 1 
			FROM sys.dm_xe_sessions 
			WHERE name = 'SQLskills_tsql_stackAction')
    DROP EVENT SESSION [SQLskills_tsql_stackAction] ON SERVER
GO
-- Create our Event Session for current Session_ID
CREATE EVENT SESSION [SQLskills_tsql_stackAction]
ON SERVER
ADD EVENT sqlserver.sql_statement_completed
(	ACTION (sqlserver.sql_text, sqlserver.tsql_stack)
	WHERE (duration > 10000000) /* exceeds 10 seconds */),
ADD EVENT sqlserver.sp_statement_completed
(	ACTION (sqlserver.sql_text, sqlserver.tsql_stack)
	WHERE (duration > 10000000) /* exceeds 10 seconds */)
ADD target package0.ring_buffer
WITH (MAX_DISPATCH_LATENCY=5SECONDS);
GO

-- Start the Event Session
ALTER EVENT SESSION [SQLskills_tsql_stackAction]
ON SERVER
STATE=START;
GO
-- Run a multi-statement Batch
SELECT * 
FROM sys.objects a, sys.objects b, sys.objects c

SELECT * 
FROM master..spt_values
WHERE type = N'P'

SELECT *
FROM INFORMATION_SCHEMA.TABLES
GO

-- Drop the Events to allow processing results
ALTER EVENT SESSION [SQLskills_tsql_stackAction]
ON SERVER
DROP EVENT sqlserver.sql_statement_completed,
DROP EVENT sqlserver.sp_statement_completed

-- Extract the Event information from the Event Session 
SELECT 
tsql_stack.value('xs:hexBinary(substring((frames/frame/@handle)[1], 3))', 'varbinary(max)'),
tsql_stack,
    event_name, 
    timestamp, 
    cpu, 
    duration, 
    reads, 
    writes, 
    state, 
    SUBSTRING(sql_text,    
                (tsql_stack.value('(frames/frame/@offsetStart)[1]', 'int')/2)+1, 
                ((CASE tsql_stack.value('(frames/frame/@offsetEnd)[1]', 'int')
                    WHEN -1 THEN DATALENGTH(sql_text) 
                    ELSE tsql_stack.value('(frames/frame/@offsetEnd)[1]', 'int') 
                  END - tsql_stack.value('(frames/frame/@offsetStart)[1]', 'int'))/2) + 1) AS statement_text_Xevents,
    SUBSTRING(st.text, (tsql_stack.value('(frames/frame/@offsetStart)[1]', 'int')/2)+1, 
        ((CASE tsql_stack.value('(frames/frame/@offsetEnd)[1]', 'int')
          WHEN -1 THEN DATALENGTH(st.text)
         ELSE tsql_stack.value('(frames/frame/@offsetEnd)[1]', 'int')
         END - tsql_stack.value('(frames/frame/@offsetStart)[1]', 'int'))/2) + 1) AS statement_text_from_cache,
    sql_text,    
    event_sequence,
    activity_id
FROM
(
    SELECT 
        event_data.value('(event/@name)[1]', 'varchar(50)') AS event_name,
        DATEADD(hh, 
            DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
            event_data.value('(event/@timestamp)[1]', 'datetime2')) AS [timestamp],
        COALESCE(event_data.value('(event/data[@name="database_id"]/value)[1]', 'int'), 
            event_data.value('(event/action[@name="database_id"]/value)[1]', 'int')) AS database_id,
        event_data.value('(event/action[@name="session_id"]/value)[1]', 'int') AS [session_id],
        event_data.value('(event/data[@name="cpu"]/value)[1]', 'int') AS [cpu],
        event_data.value('(event/data[@name="duration"]/value)[1]', 'bigint') AS [duration],
        event_data.value('(event/data[@name="reads"]/value)[1]', 'bigint') AS [reads],
        event_data.value('(event/data[@name="writes"]/value)[1]', 'bigint') AS [writes],
        event_data.value('(event/data[@name="state"]/text)[1]', 'nvarchar(4000)') AS [state],
        event_data.value('(event/data[@name="offset"]/value)[1]', 'int') AS [offset],
        event_data.value('(event/data[@name="offset_end"]/value)[1]', 'int') AS [offset_end],
        event_data.value('(event/data[@name="nest_level"]/value)[1]', 'int') AS [nest_level],
        event_data.query('(event/action[@name="tsql_stack"]/value/frames)[1]') AS [tsql_stack],
        REPLACE(event_data.value('(event/action[@name="sql_text"]/value)[1]', 'nvarchar(max)'), CHAR(10), CHAR(13)+CHAR(10)) AS [sql_text],
        event_data.value('(event/data[@name="source_database_id"]/value)[1]', 'int') AS [source_database_id],
        event_data.value('(event/data[@name="object_id"]/value)[1]', 'int') AS [object_id],
        event_data.value('(event/data[@name="object_type"]/text)[1]', 'int') AS [object_type],
        CAST(SUBSTRING(event_data.value('(event/action[@name="attach_activity_id"]/value)[1]', 'varchar(50)'), 1, 36) AS uniqueidentifier) as activity_id,
        CAST(SUBSTRING(event_data.value('(event/action[@name="attach_activity_id"]/value)[1]', 'varchar(50)'), 38, 10) AS int) as event_sequence
    FROM 
    (   SELECT XEvent.query('.') AS event_data 
        FROM 
        (    -- Cast the target_data to XML 
            SELECT CAST(target_data AS XML) AS TargetData 
            FROM sys.dm_xe_session_targets st 
            JOIN sys.dm_xe_sessions s 
                ON s.address = st.event_session_address 
            WHERE name = 'SQLskills_tsql_stackAction' 
              AND target_name = 'ring_buffer'
        ) AS Data 
        -- Split out the Event Nodes 
        CROSS APPLY TargetData.nodes ('RingBufferTarget/event') AS XEventData (XEvent)   
    ) AS tab (event_data)
) AS results
CROSS APPLY sys.dm_exec_sql_text(tsql_stack.value('xs:hexBinary(substring((frames/frame/@handle)[1], 3))', 'varbinary(max)')) AS st






/*************** Collecting a MiniDump ***************/

-- If the Event Session exists Drop it
IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_CollectMiniDump')
	DROP EVENT SESSION [SQLskills_CollectMiniDump] ON SERVER;

-- Create Event Session to find the database with most SP Recompile Events
CREATE EVENT SESSION [SQLskills_CollectMiniDump]
ON SERVER
ADD EVENT sqlserver.error_reported(
	ACTION (sqlserver.create_dump_single_thread)
	WHERE (error_number = 50000 AND
		   severity = 16 AND
		   state = 1));
GO

ALTER EVENT SESSION [SQLskills_CollectMiniDump]
ON SERVER
STATE=START;
GO

-- Open the following path in Windows 
-- C:\Program Files\Microsoft SQL Server\MSSQL10_50.MSSQLSERVER\MSSQL\Log
RAISERROR('TestMessage', 16, 1)

ALTER EVENT SESSION [SQLskills_CollectMiniDump]
ON SERVER
STATE=STOP;
GO


/*************** Inserting a Debugger Break ***************/

-- If the Event Session exists Drop it
IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_DebugBreak')
	DROP EVENT SESSION [SQLskills_DebugBreak] ON SERVER;

-- Create Event Session to find the database with most SP Recompile Events
CREATE EVENT SESSION [SQLskills_DebugBreak]
ON SERVER
ADD EVENT sqlserver.error_reported(
	ACTION (package0.debug_break)
	WHERE (error_number = 50000 AND
		   severity = 16 AND
		   state = 1));
GO

ALTER EVENT SESSION [SQLskills_DebugBreak]
ON SERVER
STATE=START;
GO

RAISERROR('TestMessage', 16, 1)
GO

ALTER EVENT SESSION [SQLskills_DebugBreak]
ON SERVER
STATE=STOP;
GO
