/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName: sql_text_Action.sql 
*
*   Summary: Demonstrates the difference between the sql_text Action and
*            the statement/batch_text columns in Extended Events.
*
*   Date: October 7, 2011 
*
*   SQL Server Versions:
*         2012
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

-- Look at available columns
SELECT * 
FROM sys.dm_xe_object_columns
WHERE object_name LIKE 'sql%completed'
  AND column_type <> 'readonly'


-- If the Event Session exists Drop it
IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_sql_text_Action')
	DROP EVENT SESSION [SQLskills_sql_text_Action] ON SERVER

-- Create Event Session to find the database with most SP Recompile Events
CREATE EVENT SESSION [SQLskills_sql_text_Action]
ON SERVER
ADD EVENT sqlserver.sql_batch_completed(
	ACTION (sqlserver.sql_text)),
ADD EVENT sqlserver.sql_statement_completed(
	ACTION (sqlserver.sql_text))
ADD TARGET package0.ring_buffer
WITH (MAX_MEMORY = 4096KB, 
   EVENT_RETENTION_MODE = ALLOW_SINGLE_EVENT_LOSS, 
   MAX_DISPATCH_LATENCY = 1 SECONDS, 
   MAX_EVENT_SIZE = 0KB, 
   MEMORY_PARTITION_MODE = NONE, 
   TRACK_CAUSALITY = ON, 
   STARTUP_STATE = OFF);
GO

ALTER EVENT SESSION [SQLskills_sql_text_Action]
ON SERVER
STATE=START;
GO

-- Perform some Tests
SELECT PASSWORD = 'bar12345!!'
SELECT 'password' as Secret
CREATE LOGIN foo WITH PASSWORD = 'bar12345!!'
SELECT 'reallylongstringwithpasswordincludedintext' AS Funny
EXEC('SELECT ''reallylongstringwithpasswordincludedintext'' AS Funny')
GO

-- Perform some Tests
SELECT PASSWORD = 'bar12345!!'
GO
SELECT 'password' as Secret
GO
CREATE LOGIN foo WITH PASSWORD = 'bar12345!!'
GO
SELECT 'reallylongstringwithpasswordincludedintext' AS Funny
GO
EXEC('SELECT ''reallylongstringwithpasswordincludedintext'' AS Funny')

ALTER EVENT SESSION [SQLskills_sql_text_Action]
ON SERVER
DROP EVENT sqlserver.sql_batch_completed,
DROP EVENT sqlserver.sql_statement_completed;
GO


-- Query the XML to get the Target Data
SELECT 
    event.value('(event/@name)[1]', 'varchar(50)') AS event_name,
    DATEADD(hh, 
            DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
            event.value('(event/@timestamp)[1]', 'datetime2')) AS [timestamp],
	ISNULL(event.value('(event/data[@name="statement"]/value)[1]', 'nvarchar(max)'),
			event.value('(event/data[@name="batch_text"]/value)[1]', 'nvarchar(max)')) AS [stmt/btch_txt],
    event.value('(event/action[@name="sql_text"]/value)[1]', 'nvarchar(max)') as [sql_text]
FROM
(   SELECT n.query('.') as event
    FROM 
    (
        SELECT CAST(target_data AS XML) AS target_data
        FROM sys.dm_xe_sessions AS s    
        JOIN sys.dm_xe_session_targets AS t
            ON s.address = t.event_session_address
        WHERE s.name = 'SQLskills_sql_text_Action'
          AND t.target_name = 'ring_buffer'
    ) AS sub
    CROSS APPLY target_data.nodes('RingBufferTarget/event') AS q(n)
) AS tab