USE [master]
GO

/*
USE [master]
GO
-- Remove any existing objects
IF EXISTS (SELECT 1 
			FROM sys.server_triggers 
			WHERE name = 'XEvents_DDLTrigger')
BEGIN
	DROP TRIGGER XEvents_DDLTrigger
	ON ALL SERVER;
END
GO

IF DB_ID('sqladmin') IS NOT NULL
BEGIN
	ALTER DATABASE sqladmin SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE sqladmin;
END
GO
*/


CREATE DATABASE sqladmin
GO
USE sqladmin
GO
CREATE TABLE dbo.XEvents_DDLOperations
( DDLEventData XML, 
  ChangeDate datetime default(current_timestamp), 
  LoginName nvarchar(256) default(suser_sname()),
  ProgramName nvarchar(256) default(program_name())
);
GO
CREATE TABLE XEvents_SessionOutputs
(
	EventSessionName nvarchar(256),
	EventName nvarchar(256),
	EventID int,
	ColumnID int,
	ColumnName nvarchar(256),
	NodeType nvarchar(10),
	DataType nvarchar(50),
	XMLLocation nvarchar(10),
	TypePrecidence int
)
GO
CREATE VIEW dbo.XETypeToSQLType 
AS
	SELECT 
		XETypeName = mv.name, 
		SQLTypeName = 'nvarchar('+CAST(MAX(LEN(mv.map_value))-(MAX(LEN(mv.map_value))%10) + 10 AS VARCHAR(4))+')',
		XMLLocation = 'text',
		TypePrecidence = 5
	FROM sys.dm_xe_object_columns oc
	LEFT JOIN sys.dm_xe_map_values mv
		ON oc.type_package_guid = mv.object_package_guid
			AND oc.type_name = mv.name
	WHERE oc.column_type = 'data'
	  AND mv.name IS NOT NULL
	GROUP BY mv.name
UNION ALL
	SELECT 
		XETypeName = o.name,
		SQLTypeName = CASE 
							WHEN TYPE_NAME IN ('int8', 'int16', 'int32', 'uint8', 
									'uint16', 'uint32', 'float32') 
								THEN 'int'
							WHEN TYPE_NAME IN ('int64', 'uint64', 'float64')
								THEN 'bigint'
							WHEN TYPE_NAME = 'boolean'
								THEN 'nvarchar(10)' --true/false returned
							WHEN TYPE_NAME = 'guid'
								THEN 'uniqueidentifier'
							ELSE 'nvarchar(4000)'
						END,
		XMLLocation = 'value',
		TypePrecidence = CASE 
							WHEN TYPE_NAME IN ('int8', 'int16', 'int32', 'uint8', 
									'uint16', 'uint32', 'float32') 
								THEN 1
							WHEN TYPE_NAME IN ('int64', 'uint64', 'float64')
								THEN 2
							WHEN TYPE_NAME = 'boolean'
								THEN 3 --true/false returned
							WHEN TYPE_NAME = 'guid'
								THEN 3
							ELSE 5
						 END
	FROM sys.dm_xe_objects o
	WHERE object_type = 'type'
	  AND TYPE_NAME != 'null'
GO
CREATE VIEW dbo.XESession_OutputsFromDMVs
AS
		-- Find a list of all the possible output columns
		SELECT 
			ses.name AS EventSessionName,
			sese.name AS EventName,
			sese.event_id AS EventID,
			oc.column_id AS ColumnID,
			oc.name AS ColumnName,
			'data' AS NodeType,
			xetst.SQLTypeName AS DataType,
			xetst.XMLLocation,
			xetst.TypePrecidence
		FROM sys.server_event_sessions AS ses
		JOIN sys.server_event_session_events AS sese
			ON ses.event_session_id = sese.event_session_id
		JOIN sys.dm_xe_packages AS p 
			ON sese.package = p.name
		JOIN sys.dm_xe_object_columns AS oc 
			ON oc.object_name = sese.name
				AND oc.object_package_guid = p.guid
		JOIN XETypeToSQLType AS xetst
			ON oc.type_name = xetst.XETypeName
		WHERE oc.column_type = 'data'
	UNION
		SELECT 
			ses.name,
			sese.name,
			sesa.event_id,
			999 as column_id,
			sesa.name,
			'action',
			xetst.SQLTypeName,
			xetst.XMLLocation,
			xetst.TypePrecidence
		FROM sys.server_event_sessions AS ses
		JOIN sys.server_event_session_events AS sese
			ON ses.event_session_id = sese.event_session_id
		JOIN sys.server_event_session_actions AS sesa
			ON ses.event_session_id = sesa.event_session_id
				AND sesa.event_id = sese.event_id
		JOIN sys.dm_xe_packages AS p
			ON sesa.package = p.name
		JOIN sys.dm_xe_objects AS o
			ON p.guid = o.package_guid
				AND sesa.name = o.name
		JOIN XETypeToSQLType AS xetst
			ON o.type_name = xetst.XETypeName
		WHERE o.object_type = 'action'
GO


-- Create a DDL Trigger to log changes to Event Sessions
CREATE TRIGGER XEvents_DDLTrigger
ON ALL SERVER 
FOR DDL_EVENT_SESSION_EVENTS
AS
BEGIN
    SET NOCOUNT ON;
	DECLARE @EventData XML = EVENTDATA();
	INSERT INTO sqladmin.dbo.XEvents_DDLOperations (DDLEventData)
	VALUES (@EventData);

	DECLARE @EventType nvarchar(256) = @EventData.value('(EVENT_INSTANCE/EventType)[1]', 'nvarchar(256)')
	DECLARE @SessionName nvarchar(256) = @EventData.value('(EVENT_INSTANCE/ObjectName)[1]', 'nvarchar(256)')

	IF @EventType = 'CREATE_EVENT_SESSION'
	BEGIN
		INSERT INTO sqladmin.dbo.XEvents_SessionOutputs 
			(EventSessionName, EventName, EventID, ColumnID, ColumnName, NodeType,
			 DataType, XMLLocation, TypePrecidence)
		SELECT EventSessionName, EventName, EventID, ColumnID, ColumnName, NodeType,
			 DataType, XMLLocation, TypePrecidence
		FROM sqladmin.dbo.XESession_OutputsFromDMVs
		WHERE EventSessionName = @SessionName
	END

	IF @EventType = 'ALTER_EVENT_SESSION'
	BEGIN
		-- Add any new outputs to the Table
		INSERT INTO sqladmin.dbo.XEvents_SessionOutputs
			(EventSessionName, EventName, EventID, ColumnID, ColumnName, NodeType,
			 DataType, XMLLocation, TypePrecidence)
		SELECT vdmv.EventSessionName, vdmv.EventName, vdmv.EventID, vdmv.ColumnID, vdmv.ColumnName, vdmv.NodeType,
			 vdmv.DataType, vdmv.XMLLocation, vdmv.TypePrecidence
		FROM sqladmin.dbo.XESession_OutputsFromDMVs vdmv
		LEFT JOIN sqladmin.dbo.XEvents_SessionOutputs xeso
			ON vdmv.EventSessionName = xeso.EventSessionName
				AND vdmv.EventName = xeso.EventName
				AND vdmv.ColumnName = vdmv.ColumnName
		WHERE vdmv.EventSessionName = @SessionName
		  AND xeso.EventSessionName IS NULL
	END

	IF @EventType = 'DROP_EVENT_SESSION'
	BEGIN
		-- Delete the Output data for the Event Session
		DELETE sqladmin.dbo.XEvents_SessionOutputs
		WHERE EventSessionName = @SessionName
	END
END
GO


-- Now lets play with an example session

CREATE EVENT SESSION [TrackResourceWaits] ON SERVER 
ADD EVENT  sqlos.wait_info
(	-- Capture the database_id, session_id, plan_handle, and sql_text
    ACTION(sqlserver.database_id,sqlserver.session_id,sqlserver.sql_text,sqlserver.plan_handle)
	WHERE
		(opcode = 1 --End Events Only
			AND duration > 100 -- had to accumulate time
			AND ((wait_type > 0 AND wait_type < 22) -- LCK_ waits
					OR (wait_type > 31 AND wait_type < 38) -- LATCH_ waits
					OR (wait_type > 47 AND wait_type < 54) -- PAGELATCH_ waits
					OR (wait_type > 63 AND wait_type < 70) -- PAGEIOLATCH_ waits
					OR (wait_type > 96 AND wait_type < 100) -- IO (Disk/Network) waits
					OR (wait_type = 107) -- RESOURCE_SEMAPHORE waits
					OR (wait_type = 113) -- SOS_WORKER waits
					OR (wait_type = 120) -- SOS_SCHEDULER_YIELD waits
					OR (wait_type = 178) -- WRITELOG waits
					OR (wait_type > 174 AND wait_type < 177) -- FCB_REPLICA_ waits
					OR (wait_type = 186) -- CMEMTHREAD waits
					OR (wait_type = 187) -- CXPACKET waits
					OR (wait_type = 207) -- TRACEWRITE waits
					OR (wait_type = 269) -- RESOURCE_SEMAPHORE_MUTEX waits
					OR (wait_type = 283) -- RESOURCE_SEMAPHORE_QUERY_COMPILE waits
					OR (wait_type = 284) -- RESOURCE_SEMAPHORE_SMALL_QUERY waits
				)
		)
)
ADD TARGET package0.ring_buffer(SET max_memory=4096)
WITH (EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,
	  MAX_DISPATCH_LATENCY=5 SECONDS)
GO


ALTER EVENT SESSION [TrackResourceWaits] ON SERVER 
STATE=START;
GO


ALTER EVENT SESSION [TrackResourceWaits] ON SERVER 
DROP EVENT  sqlos.wait_info;
GO

-- Now what is actually in our ring_buffer and how do we figure out how to query it?

SELECT *
FROM sqladmin.dbo.XEvents_SessionOutputs
WHERE EventSessionName = 'TrackResourceWaits'
GO

SELECT 
	'event_data.value(''(event/'+NodeType+'[@name="'+ColumnName+'"]/'+XMLLocation+')[1]'', '''+DataType+''') AS '+QUOTENAME(ColumnName)+',' AS XQuery,
	QUOTENAME(ColumnName)+' '+DataType+', ' AS ColumnDefinition
FROM
(
	SELECT 
		ROW_NUMBER() OVER (PARTITION BY ColumnName ORDER BY TypePrecidence DESC) AS partitionid,
		EventSessionName,
		EventID,
		ColumnID,
		ColumnName,
		NodeType,
		DataType,
		XMLLocation
	FROM sqladmin.dbo.XEvents_SessionOutputs
) AS tab
WHERE EventSessionName = 'TrackResourceWaits'
  AND partitionid = 1
ORDER BY EventID, ColumnID
GO