/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive  
*   FileName:  SQLskills_RecompilingProcedures_Event_Session_DenaliCTP3.sql
*

*   Summary: Create Event Session to track recompilations by database_id,
*            demonstrates how to shred the Event Session XML, and alter the
*            Event Session definition to change events and targets.
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         2012
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/


-- If the Event Session exists Drop it
IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_RecompilingProcedures')
	DROP EVENT SESSION [SQLskills_RecompilingProcedures] ON SERVER

-- Create Event Session to find the database with most SP Recompile Events
CREATE EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
ADD EVENT sqlserver.sp_statement_starting(
	ACTION(sqlserver.database_id)
	WHERE(state = 1) -- Recompile statement_state map_key
)
ADD TARGET package0.histogram(
	SET filtering_event_name = 'sqlserver.sp_statement_starting',
		source_type = 1, -- Action
		source = 'sqlserver.database_id'),
ADD TARGET package0.ring_buffer
WITH (MAX_DISPATCH_LATENCY=5SECONDS)
		
--Start the Event Session
ALTER EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
STATE=START	
		
		
-- RUN Workload!
EXECUTE [AdventureWorks2012].[dbo].[AnnualTop5SalesPersonByMonthlyPercent] @Year=2005
GO
EXECUTE [AdventureWorks2012].[dbo].[AnnualTop5SalesPersonByMonthlyPercent] @Year=2006
GO
EXECUTE [AdventureWorks2012].[dbo].[AnnualTop5SalesPersonByMonthlyPercent] @Year=2007
GO
EXECUTE [AdventureWorks2012].[dbo].[AnnualTop5SalesPersonByMonthlyPercent] @Year=2008

GO
DECLARE @ManagerID int
SELECT TOP 1 @ManagerID = ManagerID 
FROM AdventureWorks.HumanResources.Employee
ORDER BY NEWID()

EXECUTE AdventureWorks.dbo.uspGetManagerEmployees @ManagerID
EXECUTE AdventureWorks2012.dbo.uspGetManagerEmployees @ManagerID
EXECUTE AdventureWorks2012.dbo.uspGetEmployeeManagers @ManagerID
EXECUTE AdventureWorks.dbo.uspGetEmployeeManagers @ManagerID
GO


-- Query target data
SELECT 
    n.value('(value)[1]', 'int') AS DatabaseID,
    DB_NAME(n.value('(value)[1]', 'int')) AS DatabaseName,
    n.value('(@count)[1]', 'int') AS RecompileEventCount,
    n.value('(@trunc)[1]', 'int') AS RecompileEventsTrunc
FROM
(SELECT CAST(target_data as XML) target_data
 FROM sys.dm_xe_sessions AS s 
 JOIN sys.dm_xe_session_targets t
     ON s.address = t.event_session_address
 WHERE s.name = 'SQLskills_RecompilingProcedures'
  AND t.target_name = 'histogram' ) as tab
CROSS APPLY target_data.nodes('HistogramTarget/Slot') as q(n)

-- Modify the EVent Session 

-- Drop the sp_statement_starting event 
ALTER EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
DROP EVENT sqlserver.sp_statement_starting

-- Add the sp_statement_starting event back with predicate on database_id
ALTER EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
ADD EVENT sqlserver.sp_statement_starting(
	ACTION(sqlserver.database_id)
	WHERE(state = 1 -- Recompile statement_state map_key
		AND sqlserver.database_id = 8) -- databaseid from first capture
)

-- Drop the Bucketizer target so it can be recreated
ALTER EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
DROP TARGET package0.histogram

-- Add the Bucketizer target with 
ALTER EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
ADD TARGET package0.histogram(
	SET filtering_event_name = 'sqlserver.sp_statement_starting',
		source_type = 0, -- Event Data
		source = 'object_id')
		
/* We could have done this as we well

-- Drop the Event Session
DROP EVENT SESSION [SQLskills_RecompilingProcedures] ON SERVER

-- Create Event Session to find the specific objects recompiling
CREATE EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
ADD EVENT sqlserver.sp_statement_starting(
	ACTION(sqlserver.database_id)
	WHERE(state = 1 -- Recompile statement_state map_key
		AND sqlserver.database_id = 8) -- databaseid from first capture
)
ADD TARGET package0.histogram(
	SET filtering_event_name = 'sqlserver.sp_statement_starting',
		source_type = 0, -- Event Data
		source = 'object_id')
WITH (MAX_DISPATCH_LATENCY=5SECONDS)
		
--Start the Event Session
ALTER EVENT SESSION [SQLskills_RecompilingProcedures]
ON SERVER
STATE=START	

-- Do what makes the most sense in real life this is just a demo!  */
		

-- Query target data
SELECT 
    n.value('(value)[1]', 'int') AS ObjectID,
    OBJECT_NAME(n.value('(value)[1]', 'int'), 6) AS ObjectName,
    n.value('(@count)[1]', 'int') AS RecompileEventCount,
    n.value('(@trunc)[1]', 'int') AS RecompileEventsTrunc
FROM
(SELECT CAST(target_data as XML) target_data
 FROM sys.dm_xe_sessions AS s 
 JOIN sys.dm_xe_session_targets t
     ON s.address = t.event_session_address
 WHERE s.name = 'SQLskills_RecompilingProcedures'
  AND t.target_name = 'histogram' ) as tab
CROSS APPLY target_data.nodes('HistogramTarget/Slot') as q(n)