/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName:  event_file.sql
*
*   Summary: Demonstrates how to configure and use the file_target.
*
*   Date: October 7, 2011 
*
*   SQL Server Versions:
*         2012
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

/*  Cleanup old files
EXECUTE sp_configure 'show advanced options', 1; RECONFIGURE;
EXECUTE sp_configure 'xp_cmdshell', 1; RECONFIGURE; 
EXEC xp_cmdshell 'DEL C:\SQLskills\SQLskills_file_targetDemo*';
EXECUTE sp_configure 'xp_cmdshell', 0; RECONFIGURE;
EXECUTE sp_configure 'show advanced options', 0; RECONFIGURE;
*/


-- Target Configurable Fields
SELECT 
    oc.name AS column_name,
    oc.column_id,
    oc.type_name,
    oc.capabilities_desc,
    oc.description
FROM sys.dm_xe_packages AS p
JOIN sys.dm_xe_objects AS o 
    ON p.guid = o.package_guid
JOIN sys.dm_xe_object_columns AS oc 
    ON o.name = oc.OBJECT_NAME 
    AND o.package_guid = oc.object_package_guid
WHERE (p.capabilities IS NULL OR p.capabilities & 1 = 0)
  AND (o.capabilities IS NULL OR o.capabilities & 1 = 0)
  AND o.object_type = 'target'
  AND o.name = 'event_file'
  

IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_file_targetDemo')
	-- Drop the Event Session to cleanup Demo
	DROP EVENT SESSION [SQLskills_file_targetDemo]
	ON SERVER;

-- Create an Event Session to capture Errors Reported
CREATE EVENT SESSION [SQLskills_file_targetDemo]
ON SERVER
ADD EVENT sqlserver.module_start,
ADD EVENT sqlserver.module_end,
ADD EVENT sqlserver.sp_statement_starting,
ADD EVENT sqlserver.sp_statement_completed,
ADD EVENT sqlserver.lock_acquired
ADD TARGET package0.asynchronous_file_target(
SET filename='C:\SQLskills\SQLskills_file_targetDemo.xel',
   -- metadatafile='C:\SQLskills\SQLskills_file_targetDemo.xem',
    max_file_size=50,
    max_rollover_files=10,
    increment=5)
WITH (MAX_DISPATCH_LATENCY = 1 SECONDS);
GO

-- ALTER the Event Session and Start it.
ALTER EVENT SESSION [SQLskills_file_targetDemo]
ON SERVER
STATE=START;
GO

EXECUTE AdventureWorks2012.dbo.ExecuteLotsOfStatements @ExecutionLoopCount = 200;
GO 200

SELECT * 
FROM AdventureWorks2012.Sales.SalesOrderDetail;
GO

ALTER EVENT SESSION [SQLskills_file_targetDemo]
ON SERVER
STATE=STOP;


/*********  Read the file **********/
/***   ONLY DO THIS LIKE THIS IF REQUIRED!  THE UI IS MUCH FASTER AT SHREDDING THE DATA ***/
DECLARE @path nvarchar(260), @mdpath nvarchar(260)
  
-- Get file path
SELECT 
	@path = REPLACE(CAST(value AS NVARCHAR(512)), '.xel', '*xel')
FROM sys.server_event_sessions AS ses
JOIN sys.server_event_session_targets AS sest
         ON ses.event_session_id = sest.event_session_id
JOIN sys.server_event_session_fields AS sesf
     ON sest.event_session_id = sesf.event_session_id
    AND sest.target_id = sesf.object_id
WHERE ses.name = 'SQLskills_file_targetDemo'
  AND sest.name = 'event_file'
  AND sesf.name = 'filename';
  
-- Get metadata file path
--SELECT 
--	@mdpath = REPLACE(CAST(value AS NVARCHAR(512)), '.xem', '*xem')
--FROM sys.server_event_sessions AS ses
--JOIN sys.server_event_session_targets AS sest
--         ON ses.event_session_id = sest.event_session_id
--JOIN sys.server_event_session_fields AS sesf
--     ON sest.event_session_id = sesf.event_session_id
--    AND sest.target_id = sesf.object_id
--WHERE ses.name = 'SQLskills_file_targetDemo'
--  AND sest.name = 'event_file'
--  AND sesf.name = 'metadatafile';

-- Query the Event data from the Target
SELECT
    module_guid,
    package_guid,
    object_name,
    event_data,
    file_name,
    file_offset
FROM sys.fn_xe_file_target_read_file(@path, @mdpath, null, null);



-- Offset reading the file
/**** CHANGE THE INITIAL FILE NAME FIRST! ****/
SELECT
    module_guid,
    package_guid,
    object_name,
    event_data,
    file_name,
    file_offset
FROM sys.fn_xe_file_target_read_file(
		'C:\SQLskills\SQLskills_file_targetDemo*xel',
		'C:\SQLskills\SQLskills_file_targetDemo*xem',
		'C:\SQLskills\SQLskills_file_targetDemo_0_129626491430280000.xel',
		11534336)

-- Query the Event data from the Target.
SELECT 
    n.value('(@name)[1]', 'varchar(50)') AS event_name,
    n.value('(@package)[1]', 'varchar(50)') AS package_name,
    n.value('(@id)[1]', 'int') AS id,
    n.value('(@version)[1]', 'int') AS version,
    DATEADD(hh, 
            DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
            n.value('(@timestamp)[1]', 'datetime2')) AS [timestamp],
    n.value('(data[@name="database_id"]/value)[1]', 'int') as database_id,
    n.value('(data[@name="object_id"]/value)[1]', 'int') as object_id,
    n.value('(data[@name="duration"]/value)[1]', 'int') as duration,
    n.value('(data[@name="cpu"]/value)[1]', 'varchar(5)') as cpu,
    n.value('(data[@name="message"]/value)[1]', 'varchar(max)') as message
FROM 
(SELECT
    CAST(event_data AS XML) AS event_data
 FROM sys.fn_xe_file_target_read_file(
 		'C:\SQLskills\SQLskills_file_targetDemo*xel',
		'See its not used',
		NULL,
		NULL)
) as tab
CROSS APPLY event_data.nodes('event') as q(n)


-- Using a temp table is faster

-- Drop Results tables if they exist
IF OBJECT_ID('tempdb..#XEResults') IS NOT NULL
	DROP TABLE #XEResults 
GO
IF OBJECT_ID('tempdb..#XEResultsParsed') IS NOT NULL
	DROP TABLE #XEResultsParsed 
GO

-- Create results table to load data from XE files
CREATE TABLE #XEResults
(RowID int identity primary key, event_data XML)
GO

-- Load the event data from the file target
INSERT INTO #XEResults(event_data)
SELECT
    CAST(event_data AS XML) AS event_data
FROM sys.fn_xe_file_target_read_file(
 		'C:\SQLskills\SQLskills_file_targetDemo*xel',
		'Still Not Used :-)',
		NULL,
		NULL)
		
-- Query the Event data from the Target.
SELECT 
    event.value('(@name)[1]', 'varchar(50)') AS event_name,
    event.value('(@package)[1]', 'varchar(50)') AS package_name,
    event.value('(@id)[1]', 'int') AS id,
    event.value('(@version)[1]', 'int') AS version,
    DATEADD(hh, 
            DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
            event.value('(@timestamp)[1]', 'datetime2')) AS [timestamp],
    event.value('(data[@name="database_id"]/value)[1]', 'int') as database_id,
    event.value('(data[@name="object_id"]/value)[1]', 'int') as object_id,
    event.value('(data[@name="duration"]/value)[1]', 'int') as duration,
    event.value('(data[@name="cpu"]/value)[1]', 'varchar(5)') as cpu,
    event.value('(data[@name="message"]/value)[1]', 'varchar(max)') as message
FROM #XEResults
CROSS APPLY event_data.nodes('event') as q(event)
