/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName:  ring_buffer.sql
*
*   Summary: Demonstrates how to track what statements used a specific table
*            in a database based on object level locks.
*
*   Date: October 7, 2011 
*
*   SQL Server Versions:
*         2012
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

/*
USE AdventureWorks2012;
GO
IF OBJECT_ID('ExecuteLotsOfStatements') IS NOT NULL
BEGIN
	DROP PROCEDURE dbo.ExecuteLotsOfStatements
END
GO
CREATE PROCEDURE dbo.ExecuteLotsOfStatements
(@ExecutionLoopCount INT = 100)
AS

DECLARE @Loop INT = 0;
WHILE @Loop < @ExecutionLoopCount
BEGIN
	DECLARE @Loop2 INT
	SELECT @Loop2 = @Loop
	
	SET @Loop = @Loop+1
END

GO
*/

/*************** General Usage ***************/
IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_ring_bufferDemo')
	-- Drop the Event Session to cleanup Demo
	DROP EVENT SESSION [SQLskills_ring_bufferDemo]
	ON SERVER;

-- Create an Event Session to capture Errors Reported
CREATE EVENT SESSION [SQLskills_ring_bufferDemo]
ON SERVER
ADD EVENT sqlserver.error_reported
ADD TARGET package0.ring_buffer
WITH (MAX_DISPATCH_LATENCY = 1 SECONDS);
GO

-- ALTER the Event Session and Start it.
ALTER EVENT SESSION [SQLskills_ring_bufferDemo]
ON SERVER
STATE=START;
GO

-- SELECT from a non-existent table to create Event
SELECT *
FROM master.schema_doesnt_exist.table_doesnt_exist;
GO

-- Drop the Event to halt Event collection
ALTER EVENT SESSION [SQLskills_ring_bufferDemo]
ON SERVER
DROP EVENT sqlserver.error_reported;
GO

-- Wait for Event buffering to Target
WAITFOR DELAY '00:00:01';
GO

-- Create XML variable to hold Target Data
DECLARE @target_data XML
SELECT @target_data = CAST(target_data AS XML)
FROM sys.dm_xe_sessions AS s 
JOIN sys.dm_xe_session_targets AS t 
    ON t.event_session_address = s.address
WHERE s.name = 'SQLskills_ring_bufferDemo'
  AND t.target_name = 'ring_buffer';

-- Query XML variable to get target statistics
SELECT 
    @target_data.value('(RingBufferTarget/@eventsPerSec)[1]', 'int') AS eventsPerSec,
    @target_data.value('(RingBufferTarget/@processingTime)[1]', 'int') AS processingTime,
    @target_data.value('(RingBufferTarget/@totalEventsProcessed)[1]', 'int') AS totalEventsProcessed,
    @target_data.value('(RingBufferTarget/@eventCount)[1]', 'int') AS eventCount,
    @target_data.value('(RingBufferTarget/@droppedCount)[1]', 'int') AS droppedCount,
    @target_data.value('(RingBufferTarget/@memoryUsed)[1]', 'int') AS memoryUsed;
 
-- Query XML variable to get Event data    
SELECT 
    n.value('(@name)[1]', 'varchar(50)') AS event_name,
    n.value('(@package)[1]', 'varchar(50)') AS package_name,
    n.value('(@id)[1]', 'int') AS id,
    n.value('(@version)[1]', 'int') AS version,
    DATEADD(hh, 
            DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
            n.value('(@timestamp)[1]', 'datetime2')) AS [timestamp],
    n.value('(data[@name="error"]/value)[1]', 'int') as error,
    n.value('(data[@name="severity"]/value)[1]', 'int') as severity,
    n.value('(data[@name="duration"]/value)[1]', 'int') as state,
    n.value('(data[@name="user_defined"]/value)[1]', 'varchar(5)') as user_defined,
    n.value('(data[@name="message"]/value)[1]', 'varchar(max)') as message
FROM @target_data.nodes('RingBufferTarget/event') AS q(n);
GO

-- Drop the Event Session to cleanup Demo
DROP EVENT SESSION [SQLskills_ring_bufferDemo]
ON SERVER;

/*************** Target Options ***************/

-- Target Configurable Fields
SELECT 
    oc.name AS column_name,
    oc.column_id,
    oc.type_name,
    oc.capabilities_desc,
    oc.description
FROM sys.dm_xe_packages AS p
JOIN sys.dm_xe_objects AS o 
    ON p.guid = o.package_guid
JOIN sys.dm_xe_object_columns AS oc 
    ON o.name = oc.OBJECT_NAME 
    AND o.package_guid = oc.object_package_guid
WHERE (p.capabilities IS NULL OR p.capabilities & 1 = 0)
  AND (o.capabilities IS NULL OR o.capabilities & 1 = 0)
  AND o.object_type = 'target'
  AND o.name = 'ring_buffer';


IF EXISTS (SELECT 1 
			FROM sys.server_event_sessions 
			WHERE name = 'SQLskills_ring_bufferDemo')
	-- Drop the Event Session to cleanup Demo
	DROP EVENT SESSION [SQLskills_ring_bufferDemo]
	ON SERVER;

-- Create an Event Session to capture Errors Reported
CREATE EVENT SESSION [SQLskills_ring_bufferDemo]
ON SERVER
ADD EVENT sqlserver.module_start,
ADD EVENT sqlserver.module_end,
ADD EVENT sqlserver.sp_statement_starting,
ADD EVENT sqlserver.sp_statement_completed,
ADD EVENT sqlserver.lock_acquired
ADD TARGET package0.ring_buffer
	--(SET max_memory=128,
	--	 occurrence_number=100)
WITH (MAX_DISPATCH_LATENCY = 1 SECONDS);
GO

-- ALTER the Event Session and Start it.
ALTER EVENT SESSION [SQLskills_ring_bufferDemo]
ON SERVER
STATE=START;
GO

EXECUTE AdventureWorks2012.dbo.ExecuteLotsOfStatements @ExecutionLoopCount = 200;
GO 200

--SELECT * 
--FROM AdventureWorks2012.Sales.SalesOrderDetail;
--GO

ALTER EVENT SESSION [SQLskills_ring_bufferDemo]
ON SERVER
DROP EVENT sqlserver.module_start,
DROP EVENT sqlserver.module_end,
DROP EVENT sqlserver.sp_statement_starting,
DROP EVENT sqlserver.sp_statement_completed,
DROP EVENT sqlserver.lock_acquired

-- Create XML variable to hold Target Data
DECLARE @target_data XML
SELECT @target_data = CAST(target_data AS XML)
FROM sys.dm_xe_sessions AS s 
JOIN sys.dm_xe_session_targets AS t 
    ON t.event_session_address = s.address
WHERE s.name = 'SQLskills_ring_bufferDemo'
  AND t.target_name = 'ring_buffer';
  
-- Query XML variable to get Event data  
SELECT event_name, COUNT(*) AS occurence_count
FROM
(  
SELECT 
    n.value('(@name)[1]', 'varchar(50)') AS event_name
FROM @target_data.nodes('RingBufferTarget/event') AS q(n)
) AS tab
GROUP BY event_name;
GO

-- Uncomment Target Options and Rerun to show the last 100
-- occurrences of the module and lock event are retained


-- Uncomment the SELECT statement and rerun to show the shift
-- of occurrences to sp_statement_starting/completed at 100