/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName:  SQLskills_MonitorTempdbContention.sql
*
*   Summary: Demonstrates how to track what statements used a specific table
*            in a database based on object level locks.
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         Denali
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/
USE AdventureWorks2012
GO

IF EXISTS(SELECT * 
		  FROM sys.server_event_sessions 
		  WHERE name='SQLskills_MonitorTempdbContention')
    DROP EVENT SESSION [SQLskills_MonitorTempdbContention] ON SERVER;
GO
CREATE EVENT SESSION SQLskills_MonitorTempdbContention 
ON SERVER 
ADD EVENT sqlserver.latch_suspend_end(
    WHERE (	database_id=2 AND 
			duration>0 AND 
			(mode=3 OR 
				mode=4) AND 
			has_waiters=1 AND 
			(page_id<4 OR -- Initial allocation bitmap pages
				package0.divides_by_uint64(page_id, 8088) OR --PFS pages
				package0.divides_by_uint64(page_id, 511232) OR  --GAM Pages
				page_id=511233 OR  --2nd SGAM page 4GB-8GB
				page_id=1022465 OR --3rd SGAM page 8GB-12GB
				page_id=1533697 OR --4th SGAM page 12GB-16GB
				page_id=2044929 OR --5th SGAM page 16GB-20GB
				page_id=2556161 OR --6th SGAM page 20GB-24GB
				page_id=3067393 OR --7th SGAM page 24GB-28GB
				page_id=3578625) --8th SGAM page 28GB-32GB
			)
		)
ADD TARGET package0.ring_buffer,
ADD TARGET package0.histogram(
	SET filtering_event_name=N'sqlserver.latch_suspend_end',
		source=N'page_id',
		source_type=0);
-- Start the Event Session
ALTER EVENT SESSION [SQLskills_MonitorTempdbContention]
ON SERVER
STATE=START
GO

