USE AdventureWorks2012
GO

IF OBJECT_ID('Person.USR_GetTopDuplicateCustomer') IS NOT NULL
BEGIN
	DROP FUNCTION Person.USR_GetTopDuplicateCustomer;
END
GO
CREATE FUNCTION Person.USR_GetTopDuplicateCustomer  (@base  int)  
RETURNS @retval TABLE  
(   
      AddressLine1  NCHAR(600)  NOT NULL,   
      AddressLine2  NCHAR(600)  NULL,   
      City    NVARCHAR(30)  NOT NULL,   
      StateProvinceID  INT     NOT NULL,   
      CT     INT     NOT NULL,
	  PADDING NCHAR(2800) NOT NULL 
)  
AS  
BEGIN   
 
      INSERT INTO @retval   
      SELECT AddressLine1, AddressLine2, City, StateProvinceID, ct = COUNT(*), ''
      FROM Person.Address   
      WHERE AddressID >= @base   
      GROUP BY AddressLine1, AddressLine2, StateProvinceID, City   
      HAVING COUNT(*) > 1   
 
      RETURN  
END
GO

IF OBJECT_ID('Person.USR_IsInTop100Customers') IS NOT NULL
BEGIN
	DROP FUNCTION Person.USR_IsInTop100Customers;
END
GO
CREATE FUNCTION  Person.USR_IsInTop100Customers(@base int)
RETURNS @retval TABLE
(
	AddressID INT
)
AS
BEGIN

	INSERT INTO @retval
	SELECT TOP 100 AddressID 
	FROM Person.Address WHERE AddressID >= @base
	ORDER BY AddressID

	RETURN
END
GO

IF OBJECT_ID('Person.GetNextDuplicateCustomerSet') IS NOT NULL
BEGIN
	DROP PROCEDURE Person.GetNextDuplicateCustomerSet;
END
GO
CREATE PROCEDURE Person.GetNextDuplicateCustomerSet (@base INT)
AS
BEGIN

SELECT DISTINCT 
      (SELECT TOP 1 AddressLine1 
            FROM Person.USR_GetTopDuplicateCustomer(AddressID)) AS AddressLine1,   
      (SELECT TOP 1 ISNULL(AddressLine2, '') 
            FROM Person.USR_GetTopDuplicateCustomer(AddressID)) AS AddressLine2,   
      (SELECT TOP 1 City 
            FROM Person.USR_GetTopDuplicateCustomer(AddressID)) AS City,   
      (SELECT TOP 1 StateProvinceID 
            FROM Person.USR_GetTopDuplicateCustomer(AddressID)) AS StateProvinceID,   
      (SELECT TOP 1 CT 
            FROM Person.USR_GetTopDuplicateCustomer(AddressID)) AS ct  
FROM Person.Address   a 
WHERE AddressID IN 
      (SELECT AddressID FROM Person.USR_IsInTop100Customers(@base))  

END
GO



/*  Run Tests from here

-- This rewrite helps alleviate the PFS contention but not the GAM/SGAM contention
ALTER PROCEDURE Person.GetNextDuplicateCustomerSet (@base INT)
AS
BEGIN

SELECT TOP 1 
	t.AddressLine1,   
    ISNULL(t.AddressLine2, '') AS AddressLine2,   
    t.City,   
    t.StateProvinceID,   
    t.CT  
FROM Person.Address   a
CROSS APPLY Person.USR_GetTopDuplicateCustomer(AddressID) AS t 
WHERE AddressID IN 
      (SELECT AddressID FROM Person.USR_IsInTop1000Customers(@base))  

END

-- Script to look at the waiting tasks
SELECT
	owt.session_id,
	owt.wait_duration_ms,
	owt.wait_type,
	owt.blocking_session_id,
	owt.resource_description,
	es.program_name,
	est.text,
	est.dbid,
	eqp.query_plan,
	es.cpu_time,
	es.memory_usage
FROM sys.dm_os_waiting_tasks owt
INNER JOIN sys.dm_exec_sessions es ON
	owt.session_id = es.session_id
INNER JOIN sys.dm_exec_requests er ON
	es.session_id = er.session_id
OUTER APPLY sys.dm_exec_sql_text (er.sql_handle) est
OUTER APPLY sys.dm_exec_query_plan (er.plan_handle) eqp
WHERE es.is_user_process = 1;
GO


CREATE PROCEDURE Person.GetNextDuplicateCustomerSet_new (@base INT)
AS
BEGIN

-- get the next 1,000 addresses
SELECT TOP 1000 AddressID 
INTO #top1000Customers 
FROM Person.Address WHERE AddressID >= @base
ORDER BY AddressID
 
-- get the list of duplicate addresses
SELECT AddressLine1, AddressLine2, City, StateProvinceID, ct = COUNT(*) 
INTO #duplicateAddresses
FROM Person.Address 
WHERE AddressID IN (SELECT AddressID FROM #top1000Customers) 
GROUP BY AddressLine1, AddressLine2, StateProvinceID, City
HAVING COUNT(*) > 1
 
SELECT a.AddressID, a.AddressLine1, ISNULL(a.AddressLine2, '') AddressLine2,
      a.City, a.StateProvinceID, b.ct 
FROM Person.Address a 
JOIN #top1000Customers ta ON a.AddressID = ta.AddressID 
JOIN #duplicateAddresses b ON a.AddressLine1 = b.AddressLine1 
      AND ((a.AddressLine2 = b.AddressLine2) 
OR (a.AddressLine2 IS NULL AND b.AddressLine2 IS NULL))
      AND a.City = b.City AND a.StateProvinceID = b.StateProvinceID

END

*/