/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName:  Demo 1 - Dynamically Generate Event Session Parser XQuery.sql
*
*   Summary: Event Session for Extended Events Deep Dive
*   Summary: Dynamically generates XQuery code from the event session
*            definition stored on the server.
*			 
*   Date: October 3, 2011 
*
*   SQL Server Versions:
*         2008, 2008 R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

DECLARE @session_name SYSNAME = 'session_name'

SELECT '	event_data.value(''(event/'+node_type+'[@name="' + column_name + '"]/' + child_node_type + ')[1]'', '''+ 
CASE 
	WHEN type_name IN ('int8', 'int16', 'int32', 'uint8', 'uint16', 'uint32', 'float32') 
		THEN 'int'
	WHEN type_name IN ('int64', 'uint64', 'float64')
		THEN 'bigint'
	ELSE 'nvarchar(4000)'
	
END+''') AS ' + QUOTENAME(column_name) + ',',
QUOTENAME(column_name) + ' ' + 
CASE 
	WHEN type_name IN ('int8', 'int16', 'int32', 'uint8', 'uint16', 'uint32', 'float32') 
		THEN 'int'
	WHEN type_name IN ('int64', 'uint64', 'float64')
		THEN 'bigint'
	ELSE 'nvarchar(4000)'
END + ', '
FROM
(
SELECT 
	ROW_NUMBER() OVER (PARTITION BY column_name ORDER BY event_name, column_id) as partitionid,
	ROW_NUMBER() OVER (ORDER BY event_name, column_id) as rowid,
	event_name,
	column_name,
	type_name,
	node_type,
	child_node_type
FROM
(
-- Find a list of all the possible output columns
SELECT 
	sese.package+'.'+sese.name as event_name,
	oc.column_id,
	oc.name as column_name,
	'data' as node_type,
	oc.type_name,
	CASE WHEN o.name IS NOT NULL THEN 'text' ELSE 'value' END as child_node_type
FROM sys.server_event_sessions AS ses
JOIN sys.server_event_session_events AS sese
	ON ses.event_session_id = sese.event_session_id
JOIN sys.dm_xe_packages AS p 
	ON sese.package = p.name
JOIN sys.dm_xe_object_columns AS oc 
	ON oc.object_name = sese.name
		AND oc.object_package_guid = p.guid
LEFT OUTER JOIN sys.dm_xe_objects AS o
	ON oc.type_package_guid = o.package_guid
		AND oc.type_name = o.name
		AND o.object_type = 'map'
WHERE oc.column_type = 'data'
  AND ses.name = @session_name
UNION
SELECT 
	sese.package+'.'+sese.name as event_name,
	-9999,
	sesa.name,
	'action',
	o.type_name,
	'value'
FROM sys.server_event_sessions AS ses
JOIN sys.server_event_session_events AS sese
	ON ses.event_session_id = sese.event_session_id
JOIN sys.server_event_session_actions AS sesa
	ON ses.event_session_id = sesa.event_session_id
	AND sese.event_id = sesa.event_id
JOIN sys.dm_xe_packages AS p
	ON sesa.package = p.name
JOIN sys.dm_xe_objects AS o
	ON p.guid = o.package_guid
		AND sesa.name = o.name
WHERE o.object_type = 'action'
  AND ses.name = @session_name
) as tab
) as tab2
WHERE partitionid = 1
ORDER BY rowid ASC
